"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.StaticDeducer = void 0;
const typescript_1 = __importDefault(require("typescript"));
const path_1 = __importDefault(require("path"));
const assert_1 = __importDefault(require("assert"));
const base_1 = require("@plutolang/base");
const types_1 = require("./types");
const constants_1 = require("./constants");
const visit_var_def_1 = require("./visit-var-def");
const visit_expression_1 = require("./visit-expression");
const closure_1 = require("./closure");
const imports_1 = require("./imports");
class StaticDeducer extends base_1.core.Deducer {
    //eslint-disable-next-line @typescript-eslint/no-var-requires
    name = require(path_1.default.join(__dirname, "../package.json")).name;
    //eslint-disable-next-line @typescript-eslint/no-var-requires
    version = require(path_1.default.join(__dirname, "../package.json")).version;
    closureDir;
    constructor(args) {
        super(args);
        this.closureDir = args.closureDir;
    }
    async deduce(entrypoints) {
        if (entrypoints.length == 0) {
            throw new Error("The entrypoints is empty.");
        }
        const tsconfigPath = path_1.default.resolve("./", "tsconfig.json");
        const configFile = typescript_1.default.readConfigFile(tsconfigPath, typescript_1.default.sys.readFile);
        const configJson = typescript_1.default.parseJsonConfigFileContent(configFile.config, typescript_1.default.sys, "./");
        const archRef = await compile(entrypoints, configJson.options, {
            projectName: this.project,
            stackName: this.stack.name,
            rootpath: this.rootpath,
            closureBaseDir: this.closureDir,
        });
        return { archRef };
    }
}
exports.StaticDeducer = StaticDeducer;
async function compile(fileNames, tsOpts, ctx) {
    const program = typescript_1.default.createProgram(fileNames, tsOpts);
    const allDiagnostics = typescript_1.default.getPreEmitDiagnostics(program);
    // Emit errors
    allDiagnostics.forEach((diagnostic) => {
        if (diagnostic.file) {
            const { line, character } = typescript_1.default.getLineAndCharacterOfPosition(diagnostic.file, diagnostic.start);
            const message = typescript_1.default.flattenDiagnosticMessageText(diagnostic.messageText, "\n");
            console.log(`${diagnostic.file.fileName} (${line + 1},${character + 1}): ${message}`);
        }
        else {
            console.log(typescript_1.default.flattenDiagnosticMessageText(diagnostic.messageText, "\n"));
        }
    });
    const sourceFile = program.getSourceFile(fileNames[0]);
    const checker = program.getTypeChecker();
    let visitResult = {
        resourceRelatInfos: [],
        resourceVarInfos: [],
    };
    // Iterate through all the nodes in the global area.
    typescript_1.default.forEachChild(sourceFile, (node) => {
        const kindName = typescript_1.default.SyntaxKind[node.kind];
        switch (node.kind) {
            case typescript_1.default.SyntaxKind.VariableStatement: {
                const result = (0, visit_var_def_1.visitVariableStatement)(node, checker);
                visitResult = (0, types_1.concatVisitResult)(visitResult, result);
                break;
            }
            case typescript_1.default.SyntaxKind.ExpressionStatement: {
                const result = (0, visit_expression_1.visitExpression)(node, checker);
                visitResult = (0, types_1.concatVisitResult)(visitResult, result);
                break;
            }
            case typescript_1.default.SyntaxKind.ImportDeclaration:
            case typescript_1.default.SyntaxKind.EndOfFileToken:
            case typescript_1.default.SyntaxKind.FunctionDeclaration:
                break;
            default: {
                const { line, character } = typescript_1.default.getLineAndCharacterOfPosition(sourceFile, node.getStart());
                throw new Error(`${sourceFile.fileName} (${line + 1},${character + 1}): Sorry. Pluto doesn't currently support '${kindName}' in the global area. If you need this feature, please feel free to open an issue and let us know.`);
            }
        }
    });
    // Find all closures, and write them into the closure directory.
    storeAllClosure(visitResult.resourceVarInfos, visitResult.resourceRelatInfos, ctx);
    return buildArchRef(visitResult.resourceVarInfos, visitResult.resourceRelatInfos, ctx);
}
function storeAllClosure(resVarInfos, resRelatInfos, ctx) {
    resVarInfos.forEach((varInfo) => {
        if (varInfo.resourceConstructInfo.constructExpression !== constants_1.FN_RESOURCE_TYPE_NAME) {
            return;
        }
        const closureName = varInfo.varName;
        const imports = (0, imports_1.genImportStats)(varInfo.resourceConstructInfo.importElements).join("\n");
        const locations = varInfo.resourceConstructInfo.locations.map((loc) => {
            return {
                file: loc.file,
                depth: loc.depth,
                linenum: {
                    start: loc.start.replace(",", "-").replace(/[()]/g, ""),
                    end: loc.end.replace(",", "-").replace(/[()]/g, ""),
                },
            };
        });
        // Find all relationships that this closure is the source. Then find all resources that this
        // relationship directs to. These resources are the dependent resources.
        const dependentResources = [];
        resRelatInfos
            .filter((relatInfo) => relatInfo.fromVarName === closureName)
            .forEach((relatInfo) => {
            resVarInfos
                .filter((varInfo) => relatInfo.toVarNames.includes(varInfo.varName)) // Find the dependent resources.
                .forEach((varInfo) => {
                // Extract the imports, name, type, and parameters of the dependent resource.
                dependentResources.push({
                    imports: (0, imports_1.genImportStats)(varInfo.resourceConstructInfo.importElements).join("\n"),
                    name: varInfo.varName,
                    type: varInfo.resourceConstructInfo.constructExpression,
                    parameters: varInfo.resourceConstructInfo.parameters
                        ?.map((param) => {
                        if (param.resourceName) {
                            // TODO: Check if this parameter is a closure, or a resource. This is used to
                            // generate the closure source code. If the parameter is a closure, we use the
                            // any type to fill.
                            return "({} as any)";
                        }
                        return param.expression?.getText() ?? "undefined";
                    })
                        .join(", ") ?? "",
                });
            });
        });
        const dirpath = path_1.default.resolve(ctx.closureBaseDir, varInfo.varName);
        (0, closure_1.writeClosureToDir)(imports, locations, dependentResources, dirpath);
    });
}
function buildArchRef(resVarInfos, resRelatInfos, ctx) {
    const archClosures = [];
    const archResources = [];
    resVarInfos.forEach((varInfo) => {
        const resName = varInfo.varName;
        const resType = varInfo.resourceConstructInfo.constructExpression;
        if (resType === constants_1.FN_RESOURCE_TYPE_NAME) {
            // Closure
            const dirpath = path_1.default
                .resolve(ctx.closureBaseDir, resName)
                .replace(new RegExp(`^${ctx.rootpath}/?`), "");
            archClosures.push(new base_1.arch.Closure(resName, dirpath));
        }
        else {
            // Resource
            const resParams = varInfo.resourceConstructInfo.parameters?.map((param) => {
                return {
                    index: param.order,
                    name: param.name,
                    type: param.resourceName ? "closure" : "text",
                    value: param.resourceName ?? param.expression?.getText() ?? "undefined",
                };
            }) ?? [];
            // TODO: remove this temporary solution, fetch full quilified name of the resource type from
            // the user code.
            const tmpResType = "@plutolang/pluto." + resType;
            const resId = base_1.utils.genResourceId(ctx.projectName, ctx.stackName, tmpResType, resName);
            const res = new base_1.arch.Resource(resId, resName, tmpResType, resParams);
            archResources.push(res);
        }
    });
    const archRelats = resRelatInfos.map((relatInfo) => {
        const fromRes = archResources.find((val) => val.name == relatInfo.fromVarName) ??
            archClosures.find((val) => val.id == relatInfo.fromVarName);
        const toRes = archResources.find((val) => val.name == relatInfo.toVarNames[0]) ??
            archClosures.find((val) => val.id == relatInfo.toVarNames[0]);
        (0, assert_1.default)(fromRes !== undefined && toRes !== undefined, `${relatInfo.fromVarName} --${relatInfo.operation}--> ${relatInfo.toVarNames[0]}`);
        const fromType = fromRes instanceof base_1.arch.Closure ? "closure" : "resource";
        const toType = toRes instanceof base_1.arch.Closure ? "closure" : "resource";
        const relatType = relatInfo.type;
        const relatOp = relatInfo.operation;
        const params = relatInfo.parameters.map((param) => {
            return {
                index: param.order,
                name: param.name,
                type: param.resourceName ? "closure" : "text",
                value: param.resourceName ?? param.expression?.getText() ?? "undefined",
            };
        });
        return new base_1.arch.Relationship({ id: fromRes.id, type: fromType }, [{ id: toRes.id, type: toType }], relatType, relatOp, params);
    });
    const archRef = new base_1.arch.Architecture();
    archResources.forEach((res) => archRef.addResource(res));
    archClosures.forEach((closure) => archRef.addClosure(closure));
    archRelats.forEach((relat) => archRef.addRelationship(relat));
    return archRef;
}
//# sourceMappingURL=deducer.js.map