"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.buildImportStore = exports.genImportStats = exports.extractImportElements = exports.ImportStore = exports.ImportType = void 0;
const typescript_1 = __importDefault(require("typescript"));
/**
 * Cases:
 *  1. import { element } from "library";  // Named import
 *  2. import element, { another } from "library";  // Named import + Default import
 *  3. import * as lib from "library";  // Namespace import
 *  4. import "library";  // No import clause => Direct import
 *  5. import library from "library";  // Default import
 *  6. import { } from "library";  // Named import
 */
var ImportType;
(function (ImportType) {
    ImportType[ImportType["Named"] = 0] = "Named";
    ImportType[ImportType["Namespace"] = 1] = "Namespace";
    ImportType[ImportType["Default"] = 2] = "Default";
    ImportType[ImportType["Direct"] = 3] = "Direct";
})(ImportType || (exports.ImportType = ImportType = {}));
class ImportStore {
    // TODO: change to a more efficient data structure.
    elements;
    constructor(elements) {
        this.elements = elements ?? [];
    }
    update(elems) {
        this.elements.push(...elems);
    }
    searchElement(target) {
        for (const element of this.elements) {
            if (element.name == target) {
                return element;
            }
        }
        return;
    }
    listAllElementsByType(type) {
        return this.elements.filter((e) => e.type == type);
    }
}
exports.ImportStore = ImportStore;
/**
 * Extract all import elements from one import statement for every type.
 */
function extractImportElements(sourceFile, importStat) {
    const pkgName = importStat.moduleSpecifier.getText(sourceFile).replaceAll(/(^"|"$)/g, "");
    // For type 4, such as `import "fs"`
    if (importStat.importClause == undefined) {
        return [{ type: ImportType.Direct, name: "", package: pkgName }];
    }
    const elements = [];
    // For type 2, such as `import def, { element } from "library"`,
    // or type 5, such as `import def from "library"`
    if (importStat.importClause.name) {
        const elemName = importStat.importClause.name.escapedText.toString();
        elements.push({ type: ImportType.Default, name: elemName, package: pkgName });
    }
    // For type 5
    if (importStat.importClause.namedBindings == undefined) {
        return elements;
    }
    if (typescript_1.default.isNamedImports(importStat.importClause.namedBindings)) {
        // For type 2, type 1, such as `import { element } from "library"`,
        // or type 6, such as `import { } from "library"`
        importStat.importClause.namedBindings.elements.forEach((elem) => {
            const elemName = elem.name.escapedText.toString();
            elements.push({ type: ImportType.Named, name: elemName, package: pkgName });
        });
    }
    else if (typescript_1.default.isNamespaceImport(importStat.importClause.namedBindings)) {
        // For type 2 or type 3, such as `import * as path from "path"`
        const elemName = importStat.importClause.namedBindings.name.escapedText.toString();
        elements.push({ type: ImportType.Namespace, name: elemName, package: pkgName });
    }
    else {
        throw new Error(`Something error, niether named import nor namesapce import: ${importStat.getText(sourceFile)}`);
    }
    return elements;
}
exports.extractImportElements = extractImportElements;
function genImportStats(depElems) {
    const stats = [];
    for (const elem of depElems) {
        switch (elem.type) {
            case ImportType.Direct:
                stats.push(`import "${elem.package}";`);
                break;
            case ImportType.Default:
                stats.push(`import ${elem.name} from "${elem.package}";`);
                break;
            case ImportType.Named:
                stats.push(`import { ${elem.name} } from "${elem.package}";`);
                break;
            case ImportType.Namespace:
                stats.push(`import * as ${elem.name} from "${elem.package}";`);
                break;
            default:
                throw new Error(`Invalid import type: ${elem.type}.`);
        }
    }
    return stats;
}
exports.genImportStats = genImportStats;
const importStoreCache = new Map();
function buildImportStore(sourceFile) {
    const filename = sourceFile.fileName;
    if (importStoreCache.has(filename)) {
        return importStoreCache.get(filename);
    }
    const store = new ImportStore();
    const importStats = sourceFile.statements
        .filter(typescript_1.default.isImportDeclaration)
        .flatMap((stmt) => extractImportElements(sourceFile, stmt));
    store.update(importStats);
    importStoreCache.set(filename, store);
    return store;
}
exports.buildImportStore = buildImportStore;
//# sourceMappingURL=imports.js.map