"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.propBelongsToCapturedProps = exports.methodBelongsToInfraApi = exports.methodBelongsToClientApi = void 0;
const path_1 = require("path");
const fs_1 = require("fs");
const typescript_1 = __importDefault(require("typescript"));
const constants_1 = require("../constants");
function methodBelongsToClientApi(callExpression, typeChecker) {
    return methodBelongsToIface(callExpression, typeChecker, constants_1.PLUTO_BASE_PKG, constants_1.CLIENT_API_IFACE_NAME);
}
exports.methodBelongsToClientApi = methodBelongsToClientApi;
function methodBelongsToInfraApi(callExpression, typeChecker) {
    return methodBelongsToIface(callExpression, typeChecker, constants_1.PLUTO_BASE_PKG, constants_1.INFRA_API_IFACE_NAME);
}
exports.methodBelongsToInfraApi = methodBelongsToInfraApi;
function propBelongsToCapturedProps(propAccessExpression, typeChecker) {
    return propBelongsToIface(propAccessExpression, typeChecker, constants_1.PLUTO_BASE_PKG, constants_1.CAPTURED_PROPS_IFACE_NAME);
}
exports.propBelongsToCapturedProps = propBelongsToCapturedProps;
/**
 * Checks if the given call expression belongs to a specified interface.
 *
 * @param {ts.CallExpression} callExpression - The call expression node representing the method invocation.
 * @param {ts.TypeChecker} typeChecker - The type checker instance used to resolve types and symbols.
 * @param {string} targetIfaceName - The name of the target interface to check against.
 * @returns {boolean} - True if the method belongs to the target interface, false otherwise.
 */
function methodBelongsToIface(callExpression, typeChecker, targetPkgName, targetIfaceName) {
    // Resolve the call signature from the call expression.
    const signature = typeChecker.getResolvedSignature(callExpression);
    if (!signature) {
        return false;
    }
    // Get the declaration of the given call expression.
    const declaration = signature.getDeclaration();
    if (!declaration) {
        return false;
    }
    return declarationBelongsToIface(declaration, typeChecker, targetPkgName, targetIfaceName);
}
/**
 * Checks if the given property access expression belongs to a specified interface.
 *
 * @param {ts.PropertyAccessExpression} propAccessExpression - The property access expression node representing the method invocation.
 * @param {ts.TypeChecker} typeChecker - The type checker instance used to resolve types and symbols.
 * @param {string} targetIfaceName - The name of the target interface to check against.
 * @returns {boolean} - True if the method belongs to the target interface, false otherwise.
 */
function propBelongsToIface(propAccessExpression, typeChecker, targetPkgName, targetIfaceName) {
    const symbol = typeChecker.getSymbolAtLocation(propAccessExpression);
    const declarations = symbol?.declarations;
    if (!declarations || declarations.length === 0) {
        return false;
    }
    for (const declaration of declarations) {
        if (declarationBelongsToIface(declaration, typeChecker, targetPkgName, targetIfaceName)) {
            return true;
        }
    }
    return false;
}
function declarationBelongsToIface(declaration, typeChecker, targetPkgName, targetIfaceName) {
    if (!typescript_1.default.isClassDeclaration(declaration.parent) &&
        !typescript_1.default.isInterfaceDeclaration(declaration.parent)) {
        return false;
    }
    // Recursively inspect the type inheritance chain to determine whether the target interface exists within it.
    const typeChain = [];
    typeChain.push(typeChecker.getTypeAtLocation(declaration.parent));
    let belongsToTargetIface = false;
    while (typeChain.length > 0) {
        const currentType = typeChain.pop();
        if (!currentType.isClassOrInterface()) {
            continue;
        }
        const symbol = currentType.getSymbol();
        if (symbol && symbol.name === targetIfaceName && isSymbolFromPackage(symbol, targetPkgName)) {
            belongsToTargetIface = true;
            break;
        }
        const baseTypes = typeChecker.getBaseTypes(currentType);
        typeChain.push(...baseTypes);
    }
    return belongsToTargetIface;
}
/**
 * Check if a symbol comes from a specific package.
 *
 * @param {ts.Symbol} symbol - The symbol to check.
 * @param {string} packageName - The name of the package to check against.
 * @returns {boolean} - True if the symbol is from the package, false otherwise.
 */
function isSymbolFromPackage(symbol, packageName) {
    // Get the declaration(s) of the symbol
    const declarations = symbol.getDeclarations();
    // If there are no declarations available, the symbol does not belong to any package
    if (!declarations || declarations.length === 0) {
        return false;
    }
    // Check each declaration to see if any come from the target package
    for (const declaration of declarations) {
        const sourceFile = declaration.getSourceFile();
        const filepath = sourceFile.fileName;
        if (packageName === findPackageName(filepath)) {
            return true;
        }
    }
    // The symbol does not come from the target package
    return false;
}
function findPackageName(filepath) {
    let dirpath = filepath;
    if ((0, fs_1.statSync)(dirpath).isFile()) {
        dirpath = (0, path_1.dirname)(dirpath);
    }
    let packageName;
    while (dirpath.length >= 1) {
        const pkgJsonPath = (0, path_1.resolve)(dirpath, "package.json");
        if ((0, fs_1.existsSync)(pkgJsonPath)) {
            const pkgJsonContent = (0, fs_1.readFileSync)(pkgJsonPath).toString();
            const pkgJsonData = JSON.parse(pkgJsonContent);
            packageName = pkgJsonData["name"];
            break;
        }
        dirpath = (0, path_1.dirname)(dirpath);
    }
    return packageName;
}
//# sourceMappingURL=belong-to-iface.js.map