"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.visitBinaryExpression = exports.visitCallExpression = exports.visitExpression = void 0;
const typescript_1 = __importDefault(require("typescript"));
const base_1 = require("@plutolang/base");
const types_1 = require("./types");
const utils_1 = require("./utils");
const visit_var_def_1 = require("./visit-var-def");
const visit_calling_arguments_1 = require("./visit-calling-arguments");
/**
 * Check if this expression is doing something about resource, including:
 *   1. Create a new resource and assign it to a variable.
 *   2. Invoke a resource method.
 */
function visitExpression(parNode, checker) {
    if (process.env.DEBUG) {
        console.log(`Visit an ExpressionStatement: `, parNode.getText());
    }
    let visitResult;
    const childNode = parNode.expression;
    if (typescript_1.default.isBinaryExpression(childNode)) {
        const result = visitBinaryExpression(childNode, checker);
        visitResult = (0, types_1.concatVisitResult)(visitResult, result);
    }
    if (typescript_1.default.isCallExpression(childNode)) {
        const result = visitCallExpression(childNode, checker);
        visitResult = (0, types_1.concatVisitResult)(visitResult, result);
    }
    return visitResult;
}
exports.visitExpression = visitExpression;
function visitCallExpression(parNode, checker) {
    if (process.env.DEBUG) {
        console.log(`Visit a CallExpression: `, parNode.getText());
    }
    // Check if the expression is a function call.
    const type = checker.getTypeAtLocation(parNode.expression);
    if (!(0, utils_1.isFunctionType)(type)) {
        return;
    }
    // Get the first parameter, and check if it is a resource variable.
    let headNode = parNode.expression;
    while (typescript_1.default.isPropertyAccessExpression(headNode)) {
        headNode = headNode.expression;
    }
    if (!(0, utils_1.isResourceVar)(headNode, checker)) {
        const { line, character } = typescript_1.default.getLineAndCharacterOfPosition(parNode.getSourceFile(), parNode.getStart());
        throw new Error(`${parNode.getSourceFile().fileName} (${line + 1},${character + 1}): Currently, Pluto only allows direct access to resources, e.g. router.get(...)`);
    }
    // Get the function signature
    const signature = checker.getResolvedSignature(parNode);
    if (signature == undefined) {
        throw new Error(`Cannot get resolved signature:  + ${parNode.getText()}`);
    }
    const { closureInfos, closureDependencies, parameters } = (0, visit_calling_arguments_1.visitCallingArguments)(signature, parNode.arguments, checker);
    // Construct the relationship information for this function call.
    const accessorName = headNode.getText();
    const symbol = checker.getSymbolAtLocation(parNode.expression);
    if (symbol == undefined) {
        throw new Error("The symbol of this function call is undefined: " + parNode.getText());
    }
    const fnName = checker.symbolToString(symbol);
    const resRelatInfo = {
        fromVarName: accessorName,
        toVarNames: closureInfos.map((r) => r.varName),
        type: base_1.arch.RelatType.Create,
        operation: fnName,
        parameters: parameters,
    };
    return {
        resourceRelatInfos: [resRelatInfo].concat(...closureDependencies),
        resourceVarInfos: closureInfos,
    };
}
exports.visitCallExpression = visitCallExpression;
/**
 * Recursively visit the binary expression.
 */
function visitBinaryExpression(parNode, checker) {
    if (process.env.DEBUG) {
        console.log(`Visit a BinaryExpression: `, parNode.getText());
    }
    // const resVarInfos: ResourceVariableInfo[] = [];
    // Check if this is an assignment expression.
    // e.g. x = new MyClass();
    if (parNode.operatorToken.kind === typescript_1.default.SyntaxKind.EqualsToken) {
        const visitResult = (0, visit_var_def_1.visitAssignmentExpression)(parNode, checker);
        return visitResult;
    }
    if (parNode.operatorToken.kind !== typescript_1.default.SyntaxKind.CommaToken) {
        console.warn("The operator token is not '=' or ',', please check if the result is valid.");
    }
    let visitResult;
    const leftNode = parNode.left;
    const rightNode = parNode.right;
    // FIXME: might not be a binary expression, instead, it could be a call expression
    if (typescript_1.default.isBinaryExpression(leftNode)) {
        const result = visitBinaryExpression(leftNode, checker);
        visitResult = (0, types_1.concatVisitResult)(visitResult, result);
    }
    else {
        throw new Error("The left node of the binary expression is not a binary expression.");
    }
    if (typescript_1.default.isBinaryExpression(rightNode)) {
        const result = visitBinaryExpression(rightNode, checker);
        visitResult = (0, types_1.concatVisitResult)(visitResult, result);
    }
    else {
        throw new Error("The left node of the binary expression is not a binary expression.");
    }
    return visitResult;
}
exports.visitBinaryExpression = visitBinaryExpression;
//# sourceMappingURL=visit-expression.js.map